//Parallel port addr : default : 0x378
#define pport 0x378

//Ascii keys !
#define segA 0xE147
#define segB 0xE503
#define segC 0x2442
#define segD 0xE505
#define segE 0xA543
#define segF 0xA143
#define segG 0x6443
#define segH 0xE107
#define segI 0x0460
#define segJ 0x6404
#define segK 0xA90A
#define segL 0x2402
#define segM 0x611E
#define segN 0x6916
#define segO 0x6446
#define segP 0xA147
#define segQ 0x6C46
#define segR 0xA947
#define segS 0xC543
#define segT 0x0060
#define segU 0x6406
#define segV 0x4914
#define segW 0x7806
#define segX 0x1918
#define segY 0xC507
#define segZ 0x1548
//

unsigned int get_seg(unsigned int key);		//Reads a 16bit int ASCII code, and returns the corresponding Encoding for VFD
void send_8bit_serial_data(unsigned char data);	//Sends Serial 8bit data through our parallel port
void init_vfd(void);				//Sends VFD init commands, refer to the datasheet
void vfd_cmd(unsigned char cmd);		//Sends VFD commands in 8 bit chunks, also refer to the datasheet for more info
void vfd_send_char(unsigned int key);		//Sends a 16-bit char in 8bit chunks
void end_vfd(void);				//Same as init, but sends the commands to turn on the VFD


//Our Chars array
int seg16_ascii[] = {segA, segB, segC, segD, segE, segF, segG, segH, segI, segJ, segK, segL, segM, segN, segO, segP, segQ, segR, segS, segT, segU, segV, segW, segX, segY, segZ};


unsigned int get_seg(unsigned int key){
	if(key > 96 && key < 123){ //I only support A to Z, you'll need to modify this if you want more chars
		return (seg16_ascii[key-97]);	//Return value form array
	}else{
		return(0x0000);	//Return space if any other key
	}
}

//transmit byte serially, LSB first
void send_8bit_serial_data(unsigned char data)
{
   int i;
   int clk = 0;
   int lastBit = 0;

   usleep(1); //Required delay form the Datasheet

   //Send bits 7 to 0
   for (i = 0; i < 8; i++)
   {
	if (data & 0x01){	//Determine if we'll send a high or a low ?
		lastBit = 1;
	}else{
		lastBit = 0;
	}
 
 	//We will then pusle the clock off then on while transmitting our bit for the VFD to pick it up
	if(lastBit == 1){
		outb(0x02, pport);	//CLK LOW
		nanosleep(400);		//Delay (From Datasheet)
		outb(0x06, pport);	//CLK HIGH and send HIGH on Din !
		nanosleep(400);		//Anothre required delay
	}else{
		outb(0x00, pport);	//CLK LOW
		nanosleep(400);		//Delay
		outb(0x04, pport);	//CLK HIGH with LOW on Din !
		nanosleep(400);		//Delay
 	}
       //Shift 1 bit
       data >>= 1;
   }

}

void init_vfd(void){
//init
outb(0x05, pport);	//Send HIGH on STB and CLK
usleep(100);		//Wait a little (not required by datasheet, but it makes me feel better)

outb(0x04, pport);	//Turn off STB
usleep(1);		//Delay from Datasheet (between Strobe fall and Clk fall

vfd_cmd(0x02);		//Send 1st command CMD 0x02 (refer to the Datasheet)
outb(0x05, pport);	//Set CLK AND STB to high again
usleep(1);		//Delay 1 usec

vfd_cmd(0x40);		//Send 2nd command CMD 0x40 (Again refer to Datasheet)
outb(0x05, pport);	//CLK and STB HIGH again
usleep(1);		//Delay 1usec

vfd_cmd(0xC0);		//Send 3rd Command
outb(0x04, pport);	//Only CLK will rise this time as we will send the data next ! STB must remain low !
usleep(1);		//Delay before data sending
}

//CMD vfd
void vfd_cmd(unsigned char cmd){
	//this function just sets Clk to Hi after every cmd we send
	send_8bit_serial_data(cmd);
	outb(0x04, pport);
}

//send char
void vfd_send_char(unsigned int key){
	//This function sends 16 bits over 8 bit chunks !

	printf("%#x\r\n", key);	//Debug to console
	send_8bit_serial_data((key >> 8));	//Send Higher bits (Ex: 0xDEAD >> 8 will be 0xDE)
	outb(0x04, pport);
	send_8bit_serial_data(key & 0x00FF);	//Send lower bits (Ex: 0xBEEF & 0x00FF will be 0xEF)
	outb(0x04, pport);
}

//end
void end_vfd(void){
	//Turns on VFD (Brightness contorl too)
	vfd_cmd(0x8F);
	outb(0x05, pport); //Puts CLK and STB high again !, we won't be sending data/commands after this !
}
